-- Copyright (c) 2020 Kirazy
-- Part of Arachnophobia
--     
-- See LICENSE.md in the project directory for license information.

local sprite_fields = {
    "upper_part",
    "lower_part",
    "upper_part_shadow",
    "lower_part_shadow",
    "upper_part_water_reflection",
    "lower_part_water_reflection",
    "joint",
    "joint_shadow",
}

local allowed_legs = {
    ["leg-1"] = {10},
    ["leg-2"] = {2, 7},
    ["leg-3"] = {3, 7, 9},
    ["leg-4"] = {1, 4, 5, 8},
    ["leg-5"] = {2, 4, 6, 8, 9},
    ["leg-6"] = {2, 3, 4, 6, 7, 8},
    ["leg-7"] = {2, 3, 4, 6, 7, 8, 9},
    ["leg-8"] = {1, 2, 3, 4, 5, 6, 7, 8},
}

local leg_positions = {
    -- Right Side (Back to front)
    {leg = "spidertron-leg-1", ground_position = {2.25, -2.5}, blocking_legs = {2}, mount_position = util.by_pixel(15, -22)},
    {leg = "spidertron-leg-2", ground_position = {3, -1}, blocking_legs = {1, 3}, mount_position = util.by_pixel(23, -10)},
    {leg = "spidertron-leg-3", ground_position = {3, 1}, blocking_legs = {2, 4}, mount_position = util.by_pixel(25, 4)},
    {leg = "spidertron-leg-4", ground_position = {2.25, 2.5}, blocking_legs = {3}, mount_position = util.by_pixel(15, 17)},

    -- Left Side (Back to front)
    {leg = "spidertron-leg-5", ground_position = {-2.25, -2.5}, blocking_legs = {0}, mount_position = util.by_pixel(-15, -22)},
    {leg = "spidertron-leg-6", ground_position = {-3, -1}, blocking_legs = {0}, mount_position = util.by_pixel(-23, -10)},
    {leg = "spidertron-leg-7", ground_position = {-3, 1}, blocking_legs = {0}, mount_position = util.by_pixel(-25, 4)},
    {leg = "spidertron-leg-8", ground_position = {-2.25, 2.5}, blocking_legs = {0}, mount_position = util.by_pixel(-15, 17)},

    -- Center-line (Back to front)
    {leg = "spidertron-leg-5", ground_position = {0, -3.15}, blocking_legs = {1, 5}, mount_position = util.by_pixel(0, -24)}, -- Rear
    {leg = "spidertron-leg-8", ground_position = {0, 0}, blocking_legs = {}, mount_position = util.by_pixel(0, 0)}, -- Center
}

-- Fetch the spidertron
local spidertron = data.raw["spider-vehicle"]["spidertron"]

-- Make adjustments to the legs
for n = 1, 8 do
    -- Fetch the leg
    spider_leg = data.raw["spider-leg"]["spidertron-leg-"..n]

    -- Hide the legs
    if settings.startup["arachnophobia-leg-visibility"].value ~= "visible" then
        -- Iterate through the graphics set and eliminate the sprite fields
        for _, field in pairs(sprite_fields) do
            spider_leg.graphics_set[field] = nil
        end
    end

    -- Display laser-pointers at the feet of each leg
    if settings.startup["arachnophobia-leg-visibility"].value == "laser" then
        spider_leg.graphics_set.lower_part = {
            bottom_end = {
                layers = {
                    {
                        filename = "__arachnophobia__/graphics/spidertron-legs-lower-tip.png",
                        width = 18,
                        height = 56,
                        shift = util.by_pixel(0, -21),
                        hr_version = {
                            filename = "__arachnophobia__/graphics/hr-spidertron-legs-lower-tip.png",
                            width = 34,
                            height = 112,
                            shift = util.by_pixel(0, -21),
                            scale = 0.5,
                        }
                    },
                    -- Illuminate the laser light, pending support in base game
                    -- {
                    --     filename = "__arachnophobia__/graphics/spidertron-legs-lower-tip-light.png",
                    --     width = 18,
                    --     height = 56,
                    --     shift = util.by_pixel(0, -21),
                    --     draw_as_sprite = false,
                    --     draw_as_light = true,
                    --     hr_version = {
                    --         filename = "__arachnophobia__/graphics/hr-spidertron-legs-lower-tip-light.png",
                    --         width = 34,
                    --         height = 112,
                    --         shift = util.by_pixel(0, -21),
                    --         draw_as_sprite = false,
                    --         draw_as_light = true,
                    --     }
                    -- }
                }
            },
            bottom_end_length = 1,
            middle = util.empty_sprite(),
            middle_offset_from_bottom = 0,
            middle_offset_from_top = 0,
            top_end = util.empty_sprite(),
            top_end_length = 0
        }
    end

    -- Silence the legs
    if settings.startup["arachnophobia-mute-legs"].value == true then
        spider_leg.working_sound = nil
    end
end

-- Make adjustments to the number of legs and associated mount points
local num_legs = settings.startup["arachnophobia-number-of-legs"].value

-- Clear the legs table
spidertron.spider_engine.legs = {}

for index, leg in pairs(allowed_legs["leg-"..num_legs]) do
    table.insert(spidertron.spider_engine.legs, {
        leg = leg_positions[leg].leg,
        mount_position = leg_positions[leg].mount_position,
        ground_position = leg_positions[leg].ground_position,
        blocking_legs = leg_positions[leg].blocking_legs,
        leg_hit_the_ground_trigger = get_leg_hit_the_ground_trigger()
    })
end

-- Hide footstep particles
if settings.startup["arachnophobia-display-dust-particles"].value == false then
    for leg, _ in pairs(spidertron.spider_engine.legs) do
        spidertron.spider_engine.legs[leg].leg_hit_the_ground_trigger = nil
    end
end